// Inspired by the DuckDuckGo Privacy Extension (https://github.com/duckduckgo/duckduckgo-privacy-extension)
// Original source for this file can be found here:
// https://github.com/duckduckgo/duckduckgo-privacy-extension/tree/develop/shared/js/background

/*
 * Copyright (C) 2012, 2016 DuckDuckGo, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* For ESLint: List any global identifiers used in this file below */
/* global require log module chrome blockCounts exports */

'use strict';

const utils = require('./utils.es6');
const https = require('./https.es6');
const tabManager = require('./tab-manager.es6').tabManager;
const browserWrapper = require('./chrome-wrapper.es6');

const debugRequest = false;

/**
 * Where most of the extension work happens.
 *
 * For each request made:
 * - Upgrade http -> https where possible
 */

function handleRequest(requestData) {
  const tabId = requestData.tabId;
  // Skip requests to background tabs
  if (tabId === -1) {
    return {};
  }

  let thisTab = tabManager.get(requestData);

  // For main_frame requests: create a new tab instance whenever we either
  // don't have a tab instance for this tabId or this is a new requestId.
  //
  // Safari doesn't have specific requests for main frames
  if (requestData.type === 'main_frame' && window.chrome) {
    if (!thisTab || thisTab.requestId !== requestData.requestId) {
      const newTab = tabManager.create(requestData);

      // andrey: temporary disable this. it was letting redirect loops through on Tumblr
      // persist the last URL the tab was trying to upgrade to HTTPS
      if (thisTab && thisTab.httpsRedirects) {
        /* eslint-disable-next-line max-len */
        newTab.httpsRedirects.persistMainFrameRedirect(thisTab.httpsRedirects.getMainFrameRedirect());
      }
      thisTab = newTab;
    }
  } else {
    /**
         * Check that we have a valid tab
         * there is a chance this tab was closed before
         * we got the webrequest event
         */
    if (!(thisTab && thisTab.url && thisTab.id)) {
      return {};
    }

    /**
         * skip any broken sites
         */
    if (thisTab.site.isBroken) {
      log(`temporarily skip tracker blocking for site: ${
        utils.extractHostFromURL(thisTab.url)}\n`
              + 'more info: https://github.com/duckduckgo/content-blocking-whitelist');
      return {};
    }
  }

  /**
     * HTTPS Everywhere rules
     * If an upgrade rule is found, request is upgraded from http to https
     */

  if (!thisTab.site || !window.chrome) {
    return {};
  }

  // Skip https upgrade on broken sites
  if (thisTab.site.isBroken) {
    log(`temporarily skip https upgrades for site: ${
      utils.extractHostFromURL(thisTab.url)}\n`
              + 'more info: https://github.com/duckduckgo/content-blocking-whitelist');
    return {};
  }

  // Is this request from the tab's main frame?
  const isMainFrame = requestData.type === 'main_frame';

  // Fetch upgrade rule from https module:
  const url = https.getUpgradedUrl(requestData.url, thisTab, isMainFrame);
  if (url.toLowerCase() !== requestData.url.toLowerCase()
            && thisTab.httpsRedirects.canRedirect(requestData)) {
    console.log(`HTTPS: upgrade request url to ${url}`);
    thisTab.httpsRedirects.registerRedirect(requestData);
    // call to betafish - adblock-protect code
    blockCounts.recordOneHTTPSRedirect(tabId, requestData);

    if (isMainFrame) {
      thisTab.upgradedHttps = true;
    }
    if (utils.getUpgradeToSecureSupport()) {
      return { upgradeToSecure: true };
    }
    return { redirectUrl: url };
  } if (isMainFrame) {
    thisTab.upgradedHttps = false;
  }
  return {};
}

exports.handleRequest = handleRequest;
